/****************************************************************************************/
/*                                                                                      */
/* IBM Confidential                                                                     */
/*                                                                                      */
/* Net.Commerce Source Materials                                                        */
/*                                                                                      */
/* (C) Copyright IBM Corp. 1998.                                                        */
/*                                                                                      */
/* The source code for this program is not published or otherwise divested of its trade */
/* secrets, irrespective of what has been deposited with the U.S. Copyright Office.     */
/*                                                                                      */
/****************************************************************************************/

//=======================================================================================
// The order of the includes here is important! Do not move them around.
//=======================================================================================

#include "messages/np_msg.pch"
#include "messages/np_msg_src.h"
#include "api/api.pch"
#include <NetPerceptionsI.h>


//==============================================================================
//==============================================================================
#ifdef __TRACE_NCAPIS__
  typedef TraceYes Trace;
#else
  typedef TraceNo  Trace;
#endif
static Trace trace("NC_APIS ("__FILE__")");
//==============================================================================

#ifdef WIN32 // needed for ORBIX
#define USE_INIT
static int bootInit;
#endif

#ifdef WIN32
extern "C" char *optarg;
extern "C" int getopt(int argc, char *argv[], char *opstring);
typedef __int64 word64;
typedef unsigned __int64 uword64;
#define LONG_BIT (sizeof(long) * CHAR_BIT)
#define WORD_BIT (sizeof(uword64)/2)
#define strtoull(Str, Ptr, Base) _atoi64(Str)
#endif

#ifdef SOLARIS
typedef long long word64;
typedef unsigned long long uword64;
#endif

#define ull_to_LongLong(U,K) \
    (K).hi = ((U) >> LONG_BIT); (K).low = ((U) & ULONG_MAX);

#define LongLong_to_ull(K) \
      ((((uword64) (K).hi) << LONG_BIT) | ((uword64) (K).low))

//==============================================================================


///=========================================================
// Net Perception constants
//=========================================================

// These are default values.
static const char *_CSTR_DEFAULT_HOST = "localhost";
static const char *_CSTR_DEFAULT_SERVER = "prod";
static const char *_CSTR_DEFAULT_ROOT = "prod";
static const int _INT_DEFAULT_PORT = 9000;
static unsigned int orbix_diagnostic_level = 0;
static unsigned int orbix_max_connect_retries = 1;


//=========================================================
// Configuration Variables
//=========================================================
static const ConfField _STR_NP_ROOT("NP_ROOT");
static const ConfField _STR_NP_MODE("NP_MODE");
static const ConfField _STR_NP_HOST("NP_HOST");
static const ConfField _STR_NP_SERVER("NP_SERVER");

// Rating level for "explicitly rated"
static const long _rating_level_default_threshold = 20;

static const long _from_rank_default = 1;
static const long _to_rank_default = 10;

//=========================================================
// Configuration Variables
//=========================================================
static const ConfField _CONF_FROM_RANK ("FROM_RANK");
static const ConfField _CONF_TO_RANK ("TO_RANK");

//=========================================================
// NC Environment Parameter Names
//    must match those used by calling commands
//=========================================================
static const StringWithOwnership _PARAM_NAME_FROM_RANK("FROM_RANK");
static const StringWithOwnership _PARAM_NAME_TO_RANK("TO_RANK");
static const StringWithOwnership _PARAM_NAME_PERS_RESULT_CODE("PERS_RESULT_CODE");
static const StringWithOwnership _PARAM_NAME_FAST_LOOKUP_STRATEGY("FAST_LOOKUP_PREDICTION_STRATEGY");
static const StringWithOwnership _PARAM_NAME_FAST_LOOKUP_DIMENSION_NAME("FAST_LOOKUP_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_SUBJECT_RN("SUBJECT_RN");

//=========================================================
// NC Environment Parameter Values
//=========================================================
static const StringWithOwnership _PROMO_REC("PromoRec");




class GL_GetRecProdFrList :  public NC_Api
{
    static const ClassName _STR_ThisClass;


public:
    GL_GetRecProdFrList() 
    {
        Trace::Tracer T(_STR_CONSTRUCTOR, _STR_ThisClass);
    }

    virtual ~GL_GetRecProdFrList()
    {
        Trace::Tracer T(_STR_DESTRUCTOR, _STR_ThisClass);
    }

    void  operator delete( void* p ) { ::delete p; }

public:
    virtual bool Process(const HttpRequest& Req, HttpResponse& Res, NC_Environment& Env)
    {
        //===========================================================
        // Get Overridable function parameters from the environment.
        //===========================================================

        // Result code
        String localResultCode;

        String *envResultCode = (String *) Env.Seek(_PARAM_NAME_PERS_RESULT_CODE);
        if (envResultCode == NULL)
        {
            envResultCode = &localResultCode;
        }
        else
        {
            envResultCode->Clean();
            envResultCode->Resize(64);
        }


        // Merchant RefNum

        String* envMerchantRefNum = (String*) Env.Seek(_PARAM_NAME_MERCHANT_REF_NUM);
        if (envMerchantRefNum == NULL)
        {
            error << indent;
            error.nls(&_API_CANT_FIND_IN_ENV, _PARAM_NAME_MERCHANT_REF_NUM.c_str()) << endl;
            *envResultCode = "error";
            return false;
        }


        // 'From rank' and 'to rank'

        long nFromRank, nToRank;
        String strFromRank, strToRank;
        
        String* lvFromRank = (String*) Env.Seek(_PARAM_NAME_FROM_RANK);
        String* lvToRank = (String*) Env.Seek(_PARAM_NAME_TO_RANK);

        // I rely on short-circuit evaluation here; don't move things around.
        if(lvFromRank == NULL || lvFromRank->IsEmpty())
        {
            strFromRank = Misc::CheckConfigurationVariable(_CONF_FROM_RANK, true);
            if(strFromRank.IsEmpty())
            {
                nFromRank = _from_rank_default;
            }
            else
            {
                strFromRank.getVal(nFromRank);
            }
        }
        else
        {
            lvFromRank->getVal(nFromRank);
        }

        if(nFromRank < 1)
        {
            error << indent;
            error.nls(&_ERR_CMD_INVALID_PARAM, _PARAM_NAME_FROM_RANK.c_str()) << endl;
            *envResultCode = "error";
            return false;
        }

        // I rely on short-circuit evaluation here; don't move things around.
        if(lvToRank == NULL || lvToRank->IsEmpty())
        {
            strToRank = Misc::CheckConfigurationVariable(_CONF_TO_RANK, true);
            if(strToRank.IsEmpty())
            {
                nToRank = _to_rank_default;
            }
            else
            {
                strToRank.getVal(nToRank);
            }
        }
        else
        {
            lvToRank->getVal(nToRank);
        }

        if(nToRank < nFromRank)
        {
            error << indent;
            error << "from_rank cannot be greater than to_rank\n";
            *envResultCode = "error";
            return false;
        }
        
       // Recommended products list

        TemplateNC_PList<String, String>* envRecProdList =
        (TemplateNC_PList<String, String>*) Env.Seek(_PARAM_NAME_PRODUCT_REF_NUM_LIST);
        if(envRecProdList == NULL)
        {
            error << indent;
            error.nls(&_API_CANT_FIND_IN_ENV, _PARAM_NAME_PRODUCT_REF_NUM_LIST.c_str()) << endl;
            *envResultCode = "error";
            return false;
        }
        envRecProdList->Clean(true);


		//Get subject number (product number)
        String* subject_rn = (String*)  Env.Seek(_PARAM_NAME_SUBJECT_RN);
        if(subject_rn == NULL)
        {
            error << indent;
            error.nls(&_API_CANT_FIND_IN_ENV, _PARAM_NAME_SUBJECT_RN.c_str()) << endl;
            *envResultCode = "error";
            return false;
        }

        // Get the current shopper 

        User* envUser = (User*) Env.Seek(NC_Environment::_VAR_Shopper);
        if (envUser == NULL)
        {
            error << indent;
            error.nls(&_ERR_CANT_LOAD_SHOPPER) << endl;
            *envResultCode = "error";
            return false;
        }
        String lvShopperRefNum = envUser->getValue(User::_COL_REF_NUM);

				
        // Fast Lookup Dimension to be used
        String* envFastLookupDimensionName = (String*) Env.Seek(_PARAM_NAME_FAST_LOOKUP_DIMENSION_NAME);
        if (envFastLookupDimensionName== NULL)
        {
            error << indent; 
            error.nls(&_API_CANT_FIND_IN_ENV, _PARAM_NAME_FAST_LOOKUP_DIMENSION_NAME.c_str()) 
             << endl;
            *envResultCode = "error";
            return false;
        }

		// Fast lookup strategy used
        String* envFastLookupStrategy = (String*) Env.Seek(_PARAM_NAME_FAST_LOOKUP_STRATEGY);
        if (envFastLookupStrategy== NULL)
        {
            error << indent; 
            error.nls(&_API_CANT_FIND_IN_ENV, _PARAM_NAME_FAST_LOOKUP_STRATEGY.c_str()) 
             << endl;
            *envResultCode = "error";
            return false;
        }

		//==============================================================
        // Read NetPerception Configuration variables given in Net.Commerce
		// configuration files
        //==============================================================
        String npHOST = Misc::CheckConfigurationVariable(_STR_NP_HOST, true);
        String npROOT = Misc::CheckConfigurationVariable(_STR_NP_ROOT, true);
        String npServer = Misc::CheckConfigurationVariable(_STR_NP_SERVER, true);

		if (npServer.IsEmpty()) npServer = _CSTR_DEFAULT_SERVER;
		if (npROOT.IsEmpty()) npROOT = _CSTR_DEFAULT_ROOT;
		if (npHOST.IsEmpty()) npHOST = _CSTR_DEFAULT_HOST;

		debug << " NP HOST is : " << npHOST << endl;
		debug << " NP ROOT is : " << npROOT << endl;
		debug << " NP Server  is : " << npServer << endl;

		//==============================================================
        // Set NetPerception variables
        //==============================================================
        NetPerceptionsI::Root_var rootVar;
        NetPerceptionsI::RatingParticipantHandle subject;

 	    // update subject and object variables
	    char *tmp;
	    uword64 _key = strtoull(subject_rn->c_str(), &tmp, 0); ;
        ull_to_LongLong(_key, subject.key);

        //==============================================================
        // NetPerceptions initialization code
        //==============================================================

		// Initialize ORB	
		int argc=0;
		char *t = new char[20];
		strcpy(t,"gl_get_rec_prod_fr_list");
	    CORBA::ORB_ptr theOrbPtr = CORBA::ORB_init(argc,&t, "Orbix");

        if (CORBA::is_nil(theOrbPtr)) {
            debug << indent;
			status.nls(&_NP_ERR_NO_ORB) << endl;
	        CORBA::release(theOrbPtr);
            *envResultCode = "pers_disabled";
            return true;
        }

        theOrbPtr->setDiagnostics(orbix_diagnostic_level);
        theOrbPtr->maxConnectRetries(orbix_max_connect_retries);

		String name_server;
		name_server = npROOT << ":" << npServer;
        try {
            rootVar = NetPerceptionsI::Root::_bind(name_server.c_str(), npHOST.c_str());
        }
        catch (...) {
            debug << indent;
			status.nls(&_NP_ERR_BIND, npROOT.c_str(), name_server.c_str(), npHOST.c_str()) << endl;
	        CORBA::release(theOrbPtr);
            *envResultCode = "pers_disabled";
            return true;
        }

		
		// locate prediction starategies
        NetPerceptionsI::FastLookupPredictionStrategy_var fastlookup_strategyVar;
		try {
            fastlookup_strategyVar =
                rootVar->locate_fast_lookup_prediction_strategy(envFastLookupStrategy->c_str());
        }
        catch (...) {
            debug  <<indent;
			status.nls(&_NP_ERR_LOCATE_FASTLOOKUP_STRATEGY,envFastLookupStrategy->c_str()) << endl;
            CORBA::release(theOrbPtr);
            *envResultCode = "pers_error";
            return true;
        }


	   //===============================================================
       // Net Perception transaction begins
       //===============================================================


        //===================================================
        // Set up for Net Perception call
        //===================================================
        const char *kcsMerchantRefNum = envMerchantRefNum->c_str();
        const char *kcsShopperRefNum = lvShopperRefNum.c_str();
        bool bGLTransactionOK = true;     // Assume failure for the moment


		*envResultCode = "success";  // Assume success

		int object_count=nToRank; 

        debug <<"Merchant Ref num " << kcsMerchantRefNum << ", Shopper: " << kcsShopperRefNum <<endl;

        envRecProdList->Clean(true);
		
        // Get the prediction
        try {
			// Get the subject prediction stratergies

	           NetPerceptionsI::ContainerDimension_var fastlookup_container_dimensionVar =
                fastlookup_strategyVar->fast_lookup_subjects_container_dimension();

			   try {
                   subject.domain = fastlookup_container_dimensionVar->rater_domain()->the_key();
			   }
               catch (...) {
                   debug << indent;
				   status.nls(&_NP_ERR_SET_DOMAIN_KEY) << endl;
			   }

               NetPerceptionsI::RatingParticipantHandleList_var empty_list =
                 new NetPerceptionsI::RatingParticipantHandleList;

               NetPerceptionsI::PredictionResult_var resultVar =
                  fastlookup_strategyVar->get_prediction(subject,
                                             empty_list,
                                             object_count,
                                             empty_list);
               if (CORBA::is_nil(resultVar)) {
                   debug << indent;
				   status.nls(&_NP_DBG_NO_RECOMMENDATIONS) << endl;
  		          CORBA::release(theOrbPtr);
				  *envResultCode = "pers_error";
                  return true;
			   }

               NetPerceptionsI::PredictionResult::PredictionResults_var resultsVar =
                    resultVar->the_prediction_results();

			   //
               // Build a list of recommendations
               //
               for (CORBA::ULong j = 0; j < resultsVar->the_predictions.length(); j++) 
			   {
				  String prod;
			      prod << (long) LongLong_to_ull(resultsVar->the_predictions[j].object_handle.key);
                  envRecProdList->Append(new String(prod));
			   } 

        }
		catch (const NetPerceptionsI::BadParameter& e) {
			*envResultCode="pers_error";
            debug << indent;
			status.nls(&_NP_ERR_PRE_BAD_PARAMS) << endl;
		}
        catch (const NetPerceptionsI::DomainMismatch& e){
			*envResultCode="pers_error";
            debug << indent;
			status.nls(&_NP_ERR_PRE_DOMAIN_MISMATCH) << endl;
		}
        catch (const NetPerceptionsI::ServerError& e){
			*envResultCode="pers_error";
            debug << indent;
			status.nls(&_NP_ERR_PRE_SERVER_ERROR) << endl;
		}
        catch (...) {
            debug << indent;
			status.nls(&_NP_ERR_PRE_UNKWN_ERROR) << endl;
			*envResultCode="pers_error";
	        CORBA::release(theOrbPtr);
            return true;
        }

        //===============================================================
        // If we reach here, then the transaction is OK
        //===============================================================
        *envResultCode = "success";
        bGLTransactionOK = true;

       // return bGLTransactionOK;
       return true;
    }  // Process()
};

const ClassName GL_GetRecProdFrList::_STR_ThisClass("GL_GetRecProdFrList");

static bool X1 = NC_ApiManager::GetUniqueInstance().RegisterApi(
                       "IBM",
                       "NC",
                       "GL_GetRecProdFrList",
                       1.0,
                       new GL_GetRecProdFrList );

