/****************************************************************************************/
/*                                                                                      */
/* IBM Confidential                                                                     */
/*                                                                                      */
/* Net.Commerce Source Materials                                                        */
/*                                                                                      */
/* (C) Copyright IBM Corp. 1998.                                                        */
/*                                                                                      */
/* The source code for this program is not published or otherwise divested of its trade */
/* secrets, irrespective of what has been deposited with the U.S. Copyright Office.     */
/*                                                                                      */
/****************************************************************************************/
//////////////////////////////////////////////////////////////////////////////////////////
// Change History
//////////////////////////////////////////////////////////////////////////////////////////
// YYMMDD  F/D#   WHO    Description
// ---------------------------------------------------------------------------------------
// 981006         jbrains Initial version
//////////////////////////////////////////////////////////////////////////////////////////

#include "nc_core.pch"
#include "api/api.pch"
#include <containers/ldh_list.h>
#include "commands/np_configuration.h"
#include "messages/np_msg.pch"
#include "messages/np_msg_src.h"

/****************************************************************************************/
#ifdef __TRACE_COMMANDS__
  typedef TraceYes Trace;
#else
  typedef TraceNo  Trace;
#endif
static Trace trace("NC_COMMANDS ("__FILE__")");
/****************************************************************************************/


//////////////////////////////////////////////////////////////////////////////////////////
//
// Internal constants
//

//=========================================================
// External Parameters -- interface with URL
//=========================================================

static const StringWithOwnership paramNameURL("url");
static const StringWithOwnership paramNameMerchantRefNum("merchant_rn");
static const StringWithOwnership paramNameResultCode("cross_sell_result");
static const StringWithOwnership paramNameSubjectRefNum("subject_rn");

//=========================================================
// Internal Parameters -- interface with NCEnvironment
//=========================================================
static const StringWithOwnership _PARAM_NAME_PERS_RESULT_CODE("PERS_RESULT_CODE");
static const StringWithOwnership _PARAM_NAME_FAST_LOOKUP_STRATEGY("FAST_LOOKUP_PREDICTION_STRATEGY");
static const StringWithOwnership _PARAM_NAME_FAST_LOOKUP_DIMENSION_NAME("FAST_LOOKUP_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_SUBJECT_RN("SUBJECT_RN");

//=========================================================
// Others
//=========================================================
static const FuncName _STR_Constructor("constructor");
static const char *msgPersonalizationDisabled = "Personalization not enabled";

static const StringWithOwnership _TASK_GL_GET_REC_PROD_FR_LIST("GL_GET_REC_PROD_FR_LIST");


//static const StringWithOwnership reURL("/cgi-bin/ncommerce3/ExecMacro/demomall/cross_sell.d2w/report");

// HTML page to display if personalization not enabled
static const StringWithOwnership naURL("/demomall/pers_na.htm");

// HTML page to display if personalization error occurs
static const StringWithOwnership errorURL("/demomall/pers_error.htm");


//////////////////////////////////////////////////////////////////////////////////////////
//
//  Command CrossSellMerchant
//

class CrossSellMerchant : public NC_Command
{
private:
    static ClassName _STR_ThisClass;

    String m_merchant_rn;
    String m_result_code;
	String m_subject_rn;
	String reURL;
    TemplateNC_PList<String, String> m_product_refnum_list;

public:
    CrossSellMerchant() : m_merchant_rn(_STR_EMPTY_), m_result_code()
    {
        Trace::Tracer T(_STR_Constructor, _STR_ThisClass);
        // Do nothing!
    }

    ~CrossSellMerchant()
    {
        // Do nothing!
    }
    
    void operator delete( void* p )
    {
        ::delete p;
    }

public:
    virtual bool CheckParameters(const HttpRequest& Req, 
                                 HttpResponse& Res,
                                 NC_Environment& Env, 
                                 NC_Environment& Resources)
    {
        Trace::Tracer T(_STR_CheckParameters, _STR_ThisClass);

        Misc::CheckFieldExistenceLong(Req, paramNameMerchantRefNum, m_merchant_rn, true);
        if(m_merchant_rn.IsEmpty())
        {
            MerchantHome theMerchantHome(*DataBaseManager::GetCurrentDataBase());
            MerchantHome::SchemaList* theMerchantList;
            String whereClause;
            whereClause.Resize(STRLEN_1K);
            whereClause << Merchant::_COL_STORE_NAME << "='6ixth Avenue'";
            theMerchantList = theMerchantHome.Query(whereClause);

            if(theMerchantList == NULL)
            {
                error << indent;
                error.nls(&_API_CANT_FIND_IN_ENV, _PARAM_NAME_MERCHANT_REF_NUM.c_str()) << endl;
                m_result_code = "error";
                return false;
            }
			m_merchant_rn = theMerchantList->getHead()->getRefNum();
        }

        Misc::CheckFieldExistenceLong(Req, paramNameSubjectRefNum, m_subject_rn, true);
        if(m_subject_rn.IsEmpty())
        {
                return false;
		}
		
		if(Misc::CheckFieldExistence(Req, paramNameURL, reURL, false) == false)  return false;
        return true;
    }
        
                                 
    virtual bool Process(const HttpRequest& Req,
                         HttpResponse& Res, 
                         NC_Environment& Env)
    {

        Trace::Tracer T(_STR_Process, _STR_ThisClass);

        const ErrorMsg_Cmd* Err;

        NVPHash redirectNVP;

        //
        // We will only issue the following if personalization is enabled.
        //
        if (Misc::CheckConfigurationVariable(ConfField::_CONF_PERS_ENABLED, true) == "YES") 
        { 

			// Interest Dimension to be used
			String fast_lookup_dimension;
			fast_lookup_dimension.Clean();
			fast_lookup_dimension << FAST_LOOKUP_SUBJECT_DIMENSION_NAME << m_merchant_rn;		

			String fast_lookup_prediction_strategy;
			fast_lookup_prediction_strategy.Clean();
			fast_lookup_prediction_strategy << FAST_LOOKUP_PREDICTION_STRATEGY  << m_merchant_rn;
			

            //
            // Invoke the GL_GET_REC_PROD_FR_LIST task
            //
      
            Env.AddRef(_PARAM_NAME_MERCHANT_REF_NUM, &m_merchant_rn);
            Env.AddRef(_PARAM_NAME_SUBJECT_RN, &m_subject_rn);
            Env.AddRef(_PARAM_NAME_PRODUCT_REF_NUM_LIST, &m_product_refnum_list);
            Env.AddRef(_PARAM_NAME_PERS_RESULT_CODE, &m_result_code);
			Env.AddRef(_PARAM_NAME_FAST_LOOKUP_DIMENSION_NAME, &fast_lookup_dimension);
			Env.AddRef(_PARAM_NAME_FAST_LOOKUP_STRATEGY, &fast_lookup_prediction_strategy);
                   
            Err = NC_ApiManager::GetUniqueInstance().ProcessApi(Req, Res, Env,
                                                                _TASK_GL_GET_REC_PROD_FR_LIST,
                                                                m_merchant_rn );

            Env.Remove(_PARAM_NAME_PRODUCT_REF_NUM_LIST);
            Env.Remove(_PARAM_NAME_MERCHANT_REF_NUM);
            Env.Remove(_PARAM_NAME_SUBJECT_RN);
            Env.Remove(_PARAM_NAME_PERS_RESULT_CODE);
            Env.Remove(_PARAM_NAME_FAST_LOOKUP_DIMENSION_NAME);
            Env.Remove(_PARAM_NAME_FAST_LOOKUP_STRATEGY);

            if (Err == &_ERR_CMD_ERR_HANDLED)  // API already handled the error!
            {
                throw Err;
            }

            if (Err != NULL) // Unhandled error
            {
                error << indent;
                error.nls(&_ERR_UNHANDLED_IN_API,__FILE__,__LINE__) << endl;
                return false;
            } 
    
			
            //           
            // If personalization error occurs, then the result code will have "pers_error".  Exit and
            // display the personalization error page.
            // 
            // Actually, here we will choose to record error in the log but continue
            // with processing this command since personalization was simply sorting
            // the products in preference order.
            //
			debug << " m_result_code=" << m_result_code << endl;
            if (m_result_code == "pers_error")
            {
               debug << indent; 
               status.nls(&_NP_ERR_PERSONALIZATION_ENGINE) << endl;
            }
        }  // PERS_ENABLED
        else {
            // If personalization is disabled, we will record in log but continue
            // processing.
            error << indent; 
            error.nls(&_API_PERS_ENGINE_ERROR, msgPersonalizationDisabled) << endl;
            m_result_code = "disabled";
            debug << indent << "Redirect to [" << naURL << "]\n";
            Res.setLocation(naURL, true);
        }
 
        //           
        // If personalization error occurs, then the result code will have "pers_error".  Exit and
        // display the personalization error page.
        //
        if (m_result_code == "pers_error")
        {
               debug << indent; 
               status.nls(&_NP_ERR_PERSONALIZATION_ENGINE) << endl;
               debug << indent << "Redirect to [" << errorURL << "]\n";
               Res.setLocation(errorURL, true);
               return true;
        }
     
        String parameters("?");

        if (!m_product_refnum_list.Empty())
        {
           TemplateNC_PList<String, String>::Iterator lvI = m_product_refnum_list.getIterator();
           lvI.Begin();
           long irec = 1;
           
           while (lvI.getElement() != NULL)
           {
               String parmKey("ProdRec");
               parmKey << irec;
               String prodRefNum = lvI.getElement();
               if (!prodRefNum.IsEmpty())
               {
                  parameters << parmKey << "=" << prodRefNum << "&";
               }
               else
               {
                  break;
               }
              
               ++irec;         
               lvI.getNextElement();
           }  // while
           m_product_refnum_list.Clean(true);
        }  

        parameters << paramNameResultCode << "=" << m_result_code;
		parameters << "&" << paramNameMerchantRefNum << "=" << m_merchant_rn;
        // Construct outgoing url
   		// Set return code       
        String url(reURL);
        url << parameters;
        
        Res.setLocation(url, true);     // Load page in SSL mode.

        return true;
    }  // Process()
};

ClassName CrossSellMerchant::_STR_ThisClass("CrossSellMerchant");

static bool Z = NC_CommandManager::GetUniqueInstance().RegisterCommand("IBM",
                                                                       "NC",
                                                                       "CrossSellMerchant",
                                                                       1.0,
                                                                       new CrossSellMerchant);



