/****************************************************************************************/
/*                                                                                      */
/* IBM Confidential                                                                     */
/*                                                                                      */
/* Net.Commerce Source Materials                                                        */
/*                                                                                      */
/* (C) Copyright IBM Corp. 1998.                                                        */
/*                                                                                      */
/* The source code for this program is not published or otherwise divested of its trade */
/* secrets, irrespective of what has been deposited with the U.S. Copyright Office.     */
/*                                                                                      */
/****************************************************************************************/
//////////////////////////////////////////////////////////////////////////////////////////
// Change History
//////////////////////////////////////////////////////////////////////////////////////////
// YYMMDD  F/D#   WHO    Description
// ---------------------------------------------------------------------------------------
// 981020                Initial version
//////////////////////////////////////////////////////////////////////////////////////////

#include "nc_core.pch"
#include "api/api.pch"
#include <containers/ldh_list.h>
#include "commands/np_configuration.h"
#include "messages/np_msg.pch"
#include "messages/np_msg_src.h"


/****************************************************************************************/
#ifdef __TRACE_COMMANDS__
  typedef TraceYes Trace;
#else
  typedef TraceNo  Trace;
#endif
static Trace trace("NC_COMMANDS ("__FILE__")");
/****************************************************************************************/


//////////////////////////////////////////////////////////////////////////////////////////
//
// Internal constants
//

//=========================================================
// External Parameters -- interface with URL
//=========================================================

static const StringWithOwnership paramNameMerchantRefNum("merchant_rn");
static const StringWithOwnership paramNameResultCode("perscenter_result");
static const StringWithOwnership paramNameURL("url");


//=========================================================
// Internal Parameters -- interface with NCEnvironment
//=========================================================

static const StringWithOwnership
       _PARAM_NAME_PROFILE_SELECT_PRODUCT_LIST("PROFILE_SELECT_PRODUCT_LIST");
static const StringWithOwnership _PARAM_NAME_PERS_RESULT_CODE("PERS_RESULT_CODE");
static const StringWithOwnership _PARAM_NAME_LIKERT_DIMENSION_NAME("LIKERT_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_INTEREST_DIMENSION_NAME("INTEREST_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_LIKERT_STRATEGY("LIKERT_PREDICTION_STRATEGY");
static const StringWithOwnership _PARAM_NAME_INTEREST_STRATEGY("INTEREST_PREDICTION_STRATEGY");

//=========================================================
// Others
//=========================================================

static const FuncName _STR_Constructor("constructor");
static const char *msgPersonalizationDisabled = "Personalization not enabled";

static const StringWithOwnership _TASK_GET_REC_PROD("GL_GET_REC_PROD");
static const StringWithOwnership _TASK_SELECT_UNRATED_PRODUCTS("SELECT_UNRATED_PRODUCTS");


//static const StringWithOwnership perscenterURL("/cgi-bin/ncommerce3/ExecMacro/demomall/pers_center.d2w/report");

// HTML page to display if personalization not enabled
static const StringWithOwnership naURL("/demomall/pers_na.htm");

// HTML page to display if personalization error occurs
static const StringWithOwnership errorURL("/demomall/pers_error.htm");


//////////////////////////////////////////////////////////////////////////////////////////
//
//  Command PersonalizationCenter
//

class PersonalizationCenter : public NC_Command
{
private:
    static ClassName _STR_ThisClass;

    String m_merchant_rn;
    String m_shopper_rn;
    String m_result_code;
	String perscenterURL;
    TemplateNC_PList<String, String> m_purchased_list;
    TemplateNC_PList<String, String> m_product_refnum_list;
    
public:
    PersonalizationCenter() : m_purchased_list(true),
                               m_product_refnum_list(true),
                               m_merchant_rn(),
                               m_shopper_rn(),
                               m_result_code()
    {
        Trace::Tracer T(_STR_Constructor, _STR_ThisClass);
        // Do nothing!
    }

    ~PersonalizationCenter()
    {
        // Do nothing!
    }
    
    void operator delete( void* p )
    {
        ::delete p;
    }

public:
    virtual bool CheckParameters(const HttpRequest& Req, 
                                 HttpResponse& Res,
                                 NC_Environment& Env, 
                                 NC_Environment& Resources)
    {
       
		Misc::CheckFieldExistenceLong(Req, paramNameMerchantRefNum, m_merchant_rn, true);
        if(m_merchant_rn.IsEmpty())
        {
            MerchantHome theMerchantHome(*DataBaseManager::GetCurrentDataBase());
            MerchantHome::SchemaList* theMerchantList;
            String whereClause;
            whereClause.Resize(STRLEN_1K);
            whereClause << Merchant::_COL_STORE_NAME << "='6ixth Avenue'";
            theMerchantList = theMerchantHome.Query(whereClause);

            if(theMerchantList == NULL)
            {
                error << indent;
                error.nls(&_API_CANT_FIND_IN_ENV, _PARAM_NAME_MERCHANT_REF_NUM.c_str()) << endl;
                m_result_code = "error";
                return false;
            }
			m_merchant_rn = theMerchantList->getHead()->getRefNum();
        }
		
        if(Misc::CheckFieldExistence(Req, paramNameURL, perscenterURL, false) == false)  return false;

        return true;
    }

                                 
    virtual bool Process(const HttpRequest& Req,
                         HttpResponse& Res, 
                         NC_Environment& Env)
    {
        Trace::Tracer T(_STR_Process, _STR_ThisClass);

        NVPHash redirectNVP;

        if (Misc::CheckConfigurationVariable(ConfField::_CONF_PERS_ENABLED, true) == "YES") 
        {
            const ErrorMsg_Cmd* Err;

            //=================================================================
            // Check for products which have been purchased, but not explicitly
            // rated.
            //=================================================================


            //
            // Get the current shopper
            //
        
            User* envUser = (User*) Env.Seek(NC_Environment::_VAR_Shopper);
            if (envUser == NULL)
            {
                error << indent;
                error.nls(&_ERR_CANT_LOAD_SHOPPER) << endl;
                m_result_code = "error";
                return false;
            }
            m_shopper_rn = envUser->getValue(User::_COL_REF_NUM);

            //
            // Get the merchant refnum for 6ixth Avenue
            //

            if(m_merchant_rn.IsEmpty())
            {
                String *envMerchantRefNum = (String*) Env.Seek(_PARAM_NAME_MERCHANT_REF_NUM);
                if(envMerchantRefNum == NULL)
                {
                    // Go to the database, then.
                    MerchantHome theMerchantHome(*DataBaseManager::GetCurrentDataBase());
                    MerchantHome::SchemaList* theMerchantList;
                    String whereClause;
                    whereClause.Resize(STRLEN_1K);
                    whereClause << Merchant::_COL_STORE_NAME << "='6ixth Avenue'";
                    theMerchantList = theMerchantHome.Query(whereClause);

                    if(theMerchantList == NULL)
                    {
                        error << indent;
                        error.nls(&_API_CANT_FIND_IN_ENV, _PARAM_NAME_MERCHANT_REF_NUM.c_str()) << endl;
                        m_result_code = "error";
                        return false;
                    }
        			m_merchant_rn = theMerchantList->getHead()->getRefNum();
                }
                else {
                    m_merchant_rn = *envMerchantRefNum;
                }
            }


            //
            // Get the list of products purchased by the current shopper
            //
            // We need the SQL query:
            // select distinct shipto.stprnbr from shipto, orders
            // where shipto.stornbr=orders.orrfnbr and orders.orshnbr=$(shopperRefNum)
            // and orders.ormenbr=$(6ixthAvenueMerchantRefNum)
            //

            DataBase* DB = DataBaseManager::GetCurrentDataBase();

            
            String orderItemQuery;
            orderItemQuery.Resize(STRLEN_1K);

            orderItemQuery << "SELECT DISTINCT shipto.stprnbr, product.prprfnbr FROM "
             << "product, shipto, orders where shipto.stornbr=orders.orrfnbr and orders.orshnbr="
             << m_shopper_rn << " and product.prrfnbr=shipto.stprnbr and orders.ormenbr="
             << m_merchant_rn;

            SQL orderItemSQL(*DB, orderItemQuery);
            Row orderItemRow;
            long sqlError;

            m_purchased_list.Clean(true);
            while( (sqlError = orderItemSQL.getNextRow(orderItemRow)) == ERR_DB_NO_ERROR )
            {
                String thisProductRefNum = orderItemRow.getCol(1).Trim();
                String thisParentRefNum = orderItemRow.getCol(2).Trim();

                String parentItemQuery;
            
                bool bParentRefNumExists = true;
                while(!thisParentRefNum.IsEmpty())
                {
                    parentItemQuery.Clean();
                    parentItemQuery.Resize(STRLEN_1K);
                    parentItemQuery << "SELECT product.prprfnbr FROM product ";
                    parentItemQuery << "where product.prrfnbr=" << thisParentRefNum;
                    SQL parentItemSQL(*DB, parentItemQuery);
                    Row parentItemRow;

                    if((sqlError = parentItemSQL.getNextRow(parentItemRow)) == ERR_DB_NO_ERROR)
                    {
                        thisProductRefNum = thisParentRefNum;
                        thisParentRefNum = parentItemRow.getCol(1).Trim();
                    }
                    else
                    {
                        error << indent << "Could not find parent product reference "
                            << "number for product " << thisParentRefNum;
                        bParentRefNumExists = false;
                        m_result_code = "error";
                        break;
                        // return false;
                    }
                }
                if(bParentRefNumExists)  m_purchased_list.Append( new String( thisProductRefNum ) );
            }

			
			// Likert Dimension to be used
			String likert_dimension;
			likert_dimension.Clean();
			likert_dimension << LIKERT_DIMENSION_NAME << m_merchant_rn;

			// Interest Dimension to be used
			String interest_dimension;
			interest_dimension.Clean();
			interest_dimension << INTEREST_DIMENSION_NAME << m_merchant_rn;		

			String likert_prediction_strategy;
			likert_prediction_strategy.Clean();
			likert_prediction_strategy << LIKERT_PREDICTION_STRATEGY << m_merchant_rn;
 
			String interest_prediction_strategy;
			interest_prediction_strategy.Clean();
			interest_prediction_strategy << INTEREST_PREDICTION_STRATEGY << m_merchant_rn;


            // Any items removed from the purchased list must be deleted by the OF.      
            Env.AddRef(_PARAM_NAME_MERCHANT_REF_NUM, &m_merchant_rn);
            Env.AddRef(_PARAM_NAME_PROFILE_SELECT_PRODUCT_LIST, &m_purchased_list);
            Env.AddRef(_PARAM_NAME_PERS_RESULT_CODE, &m_result_code);
			Env.AddRef(_PARAM_NAME_LIKERT_DIMENSION_NAME, &likert_dimension);
                   
            Err = NC_ApiManager::GetUniqueInstance().ProcessApi(Req, Res, Env,
                                                  _TASK_SELECT_UNRATED_PRODUCTS,
                                                  m_merchant_rn );

            Env.Remove(_PARAM_NAME_PROFILE_SELECT_PRODUCT_LIST);
            Env.Remove(_PARAM_NAME_MERCHANT_REF_NUM);
            Env.Remove(_PARAM_NAME_PERS_RESULT_CODE);
            Env.Remove(_PARAM_NAME_LIKERT_DIMENSION_NAME);

                   
            if (Err == &_ERR_CMD_ERR_HANDLED)  // API already handled the error!
            {
                throw Err;
            }

            if (Err != NULL) // Unhandled error
            {
                error << indent;
                error.nls(&_ERR_UNHANDLED_IN_API,__FILE__,__LINE__) << endl;
                return false;
            } 


            //           
            // If personalization error occurs, then the result code will have "pers_error".  Exit and
            // display the personalization error page.
            //
            if (m_result_code == "pers_error")
            {
               debug << indent; 
               status.nls(&_NP_ERR_PERSONALIZATION_ENGINE) << endl;
               status << indent << "Redirect to [" << errorURL << "]\n";
               Res.setLocation(errorURL, true);
               return true;
            }
     
   

            //
            // Now m_purchased_list contains UNRATED, purchased items
            //



            //
            // Get recommended products
            //                 
            Env.AddRef(_PARAM_NAME_MERCHANT_REF_NUM, &m_merchant_rn);
            Env.AddRef(_PARAM_NAME_PRODUCT_REF_NUM_LIST, &m_product_refnum_list);
            Env.AddRef(_PARAM_NAME_PERS_RESULT_CODE, &m_result_code);
			Env.AddRef(_PARAM_NAME_LIKERT_DIMENSION_NAME, &likert_dimension);
			Env.AddRef(_PARAM_NAME_INTEREST_DIMENSION_NAME, &interest_dimension);
			Env.AddRef(_PARAM_NAME_LIKERT_STRATEGY, &likert_prediction_strategy);
			Env.AddRef(_PARAM_NAME_INTEREST_STRATEGY, &interest_prediction_strategy);

                   
            Err = NC_ApiManager::GetUniqueInstance().ProcessApi(Req, Res, Env,
                                                          _TASK_GET_REC_PROD,
                                                          m_merchant_rn );

            Env.Remove(_PARAM_NAME_MERCHANT_REF_NUM);
            Env.Remove(_PARAM_NAME_PRODUCT_REF_NUM_LIST);
            Env.Remove(_PARAM_NAME_PERS_RESULT_CODE);
            Env.Remove(_PARAM_NAME_LIKERT_DIMENSION_NAME);
            Env.Remove(_PARAM_NAME_INTEREST_DIMENSION_NAME);
            Env.Remove(_PARAM_NAME_LIKERT_STRATEGY);
            Env.Remove(_PARAM_NAME_INTEREST_STRATEGY);
                   
            if (Err == &_ERR_CMD_ERR_HANDLED)  // API already handled the error!
            {
                throw Err;
            }

            if (Err != NULL) // Unhandled error
            {
                error.nls(&_ERR_UNHANDLED_IN_API,__FILE__,__LINE__) << endl;
                return false;
            } 



            //           
            // If personalization error occurs, then the result code will have "pers_error".  Exit and
            // display the personalization error page.
            //
            if (m_result_code == "pers_error")
            {
               debug << indent; 
               status.nls(&_NP_ERR_PERSONALIZATION_ENGINE) << endl;
               status << indent << "Redirect to [" << errorURL << "]\n";
               Res.setLocation(errorURL, true);
               return true;
            }
     
 


            //
            // Build the parameter list
            //

            String parameters("?");

            if (!m_product_refnum_list.Empty())
            {
               TemplateNC_PList<String, String>::Iterator lvI = m_product_refnum_list.getIterator();
               lvI.Begin();
               long irec = 1;
           
               while (lvI.getElement() != NULL)
               {
                  String parmKey("ProdRec");
                  parmKey << irec;
                  String prodRefNum = lvI.getElement();
                  if (!prodRefNum.IsEmpty())
                  {
                     parameters << parmKey << "=" << prodRefNum << "&";
                  }
                  else
                  {
                     break;
                  }
              
                  ++irec;         
                  lvI.getNextElement();
               }  // while
               m_product_refnum_list.Clean(true);
            }  
        
            //m_result_code = "success";

            if(m_purchased_list.Count() != 0)
            {
                TemplateNC_PList<String, String>::Iterator iUnratedProducts(&m_purchased_list);
                int product_number;
                for(iUnratedProducts.Begin(), product_number = 1;
                    *iUnratedProducts != NULL && product_number <= 10;
                    ++iUnratedProducts, ++product_number)
                {
                    parameters << "special" << product_number << "="
                     << *(iUnratedProducts.getElement()) << "&";
                }
            }
            parameters << paramNameResultCode << "=" << m_result_code;
			parameters << "&" << paramNameMerchantRefNum << "=" << m_merchant_rn;

            // Construct outgoing url
   		    // Set return code        }  // PERS_ENABLED==YES			
            String url(perscenterURL);
            url << parameters;
          
            Res.setLocation(url, true);     // Load page in SSL mode.
        }
        else
        {
            error << indent; 
            error.nls(&_API_PERS_ENGINE_ERROR, msgPersonalizationDisabled) << endl;
            m_result_code = "disabled";
            debug << indent << "Redirect to [" << naURL << "]\n";
            Res.setLocation(naURL, true);
        }
        return true;
    }  //Process()
};

ClassName PersonalizationCenter::_STR_ThisClass("PersonalizationCenter");

static bool Z = NC_CommandManager::GetUniqueInstance().RegisterCommand(
              "IBM",
              "NC",
              "PersonalizationCenter",
              1.0,
              new PersonalizationCenter);



