/****************************************************************************************/
/*                                                                                      */
/* IBM Confidential                                                                     */
/*                                                                                      */
/* Net.Commerce Source Materials                                                        */
/*                                                                                      */
/* (C) Copyright IBM Corp. 1998.                                                        */
/*                                                                                      */
/* The source code for this program is not published or otherwise divested of its trade */
/* secrets, irrespective of what has been deposited with the U.S. Copyright Office.     */
/*                                                                                      */
/****************************************************************************************/

#include "nc_core.pch"
#include "api/api.pch"
#include <containers/ldh_list.h>
#include "commands/np_configuration.h"
#include "messages/np_msg.pch"
#include "messages/np_msg_src.h"

/****************************************************************************************/
#ifdef __TRACE_COMMANDS__
  typedef TraceYes Trace;
#else
  typedef TraceNo  Trace;
#endif
static Trace trace("NC_COMMANDS ("__FILE__")");
/****************************************************************************************/


//////////////////////////////////////////////////////////////////////////////////////////
//
// Internal constants
//

//============================================================================
// External Parameters -- interface with URL
//============================================================================

static const StringWithOwnership paramNameMerchantRefNum("merchant_rn");
static const StringWithOwnership paramNameProductRefNum("product_rn");
static const StringWithOwnership paramNameProductsRated("products_rated");
static const StringWithOwnership paramNameRating("rating");
static const StringWithOwnership paramNameURL("url");
static const StringWithOwnership paramNameResultCode("rate_result");

//============================================================================
// Internal Parameters -- interface with NCEnvironment
//============================================================================

static const StringWithOwnership _PARAM_NAME_BEHAVIOR_LIST("BEHAVIOR_LIST");
static const StringWithOwnership _PARAM_NAME_PERS_RESULT_CODE("PERS_RESULT_CODE");
static const StringWithOwnership _PARAM_NAME_LIKERT_DIMENSION_NAME("LIKERT_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_INTEREST_DIMENSION_NAME("INTEREST_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_LIKERT_RATING_STRATEGY("LIKERT_RATING_STRATEGY");
static const StringWithOwnership _PARAM_NAME_INTEREST_RATING_STRATEGY("INTEREST_RATING_STRATEGY");

//============================================================================
// Others
//============================================================================

static const FuncName _STR_Constructor("<constructor>");
static const String strEmptyMerchantRefNum("EMPTY");
static const char *msgPersonalizationDisabled = "Personalization not enabled";


// HTML page to display if personalization not enabled
static const StringWithOwnership naURL("/demomall/pers_na.htm");

// HTML page to display if personalization error occurs
static const StringWithOwnership errorURL("/demomall/pers_error.htm");

//////////////////////////////////////////////////////////////////////////////////////////
//
//  Command RateProduct
//

class RateProduct : public NC_Command
{
private:
    static ClassName _STR_ThisClass;

    TemplateNC_PList<String, String> m_product_rn_list;
    TemplateNC_PList<String, String> m_customer_rating_list;
    String m_merchant_rn;
    String m_products_rated;
    String m_url;
    String m_result_code;

public:
    RateProduct() : m_customer_rating_list(true), m_product_rn_list(true), 
                    m_merchant_rn(strEmptyMerchantRefNum), m_products_rated("0"),
                    m_url(), m_result_code("nothing")
    {
        Trace::Tracer T(_STR_Constructor, _STR_ThisClass);
        // Do nothing!
    }

    ~RateProduct()
    {
        // Do nothing!
    }
    
    void operator delete( void* p )
    {
        ::delete p;
    }

public:
    virtual bool CheckParameters(const HttpRequest& Req, 
                                 HttpResponse& Res,
                                 NC_Environment& Env, 
                                 NC_Environment& Resources)
    {
        Trace::Tracer T(_STR_CheckParameters, _STR_ThisClass);
        String paramName;
        String thisParameter;

        m_product_rn_list.Clean(true);
        m_customer_rating_list.Clean(true);

        if(Misc::CheckFieldExistenceLong(Req, paramNameProductsRated,
                                         m_products_rated, false) == false)  return false;

        
        long nProductsRated;
        if(m_products_rated.getVal(nProductsRated) == false)  return false;

        paramName.Resize(64);
        thisParameter.Resize(64);

        long i = 0;
        for(i = 1; i <= nProductsRated; i++) 
        {
            long thisProductRefNum;
            
            paramName.Clean();

            paramName << paramNameProductRefNum << "_" << i;

            if(Misc::CheckFieldExistenceLong(Req, paramName,
                                             thisParameter, false) == false)  return false;

            thisParameter.getVal(thisProductRefNum);
            if(thisProductRefNum < 0)
            {
                error << indent;
                error.nls(&_ERR_CMD_INVALID_PARAM, paramName.c_str()) << endl;
                m_product_rn_list.Clean(true);
                m_customer_rating_list.Clean(true);
                return false;
            }
                
            m_product_rn_list.Append(new String(thisParameter));

            paramName.Clean();
            thisParameter.Clean();

            paramName << paramNameRating << "_" << i;
            
            if(Misc::CheckFieldExistence(Req, paramName,
                                         thisParameter, false) == false)  return false;

            m_customer_rating_list.Append(new String(thisParameter));
        }

        if(Misc::CheckFieldExistence(Req, paramNameURL,
                                     m_url, false) == false)  return false;

        // Merchant RefNum is an optional parameter
        Misc::CheckFieldExistenceLong(Req, paramNameMerchantRefNum,
                                      m_merchant_rn, false);

        return true;
    }
        
                                 
    virtual bool Process(const HttpRequest& Req,
                         HttpResponse& Res, 
                         NC_Environment& Env)
    {
        Trace::Tracer T(_STR_Process, _STR_ThisClass);

        NVPHash redirectNVP;
        NVPHash nvpEmpty;

        m_result_code = "nothing";  // Initialising in the constructor is not enough
        
        if (Misc::CheckConfigurationVariable(ConfField::_CONF_PERS_ENABLED, true) == "YES")
        {
            const ErrorMsg_Cmd* Err;

            if(m_merchant_rn == strEmptyMerchantRefNum)
            {
                // Get the merchant refnum based on the first product passed
                // in from the URL.
                
                ProductHome aProductHome(*DataBaseManager::GetCurrentDataBase());
                String *firstProductRefNum = m_product_rn_list.getHead();

                // We are guaranteed that firstProductRefNum points to a
                // valid product refnum.

                Product *firstProduct = aProductHome.Lookup(*firstProductRefNum, nvpEmpty);
                if( firstProduct == NULL )
                {
                    error << indent;
                    error.nls(&_ERR_PROD_NOT_EXISTING, firstProductRefNum->c_str()) << endl;
                    m_result_code = "error";
                    return false;
                }
                m_merchant_rn = firstProduct->getMerchantRefNum();
            }

			// Dimension to be used
			String likert_dimension;
			likert_dimension.Clean();
			likert_dimension << LIKERT_DIMENSION_NAME << m_merchant_rn;

			String interest_dimension;
			interest_dimension.Clean();
			interest_dimension << INTEREST_DIMENSION_NAME  << m_merchant_rn;

			String likert_rating_strategy;
			likert_rating_strategy.Clean();
			likert_rating_strategy << LIKERT_RATING_STRATEGY << m_merchant_rn;
 
			String interest_rating_strategy;
			interest_rating_strategy.Clean();
			interest_rating_strategy << INTEREST_RATING_STRATEGY << m_merchant_rn;
                   
            Env.AddRef(_PARAM_NAME_MERCHANT_REF_NUM, &m_merchant_rn);
            Env.AddRef(_PARAM_NAME_PRODUCT_REF_NUM_LIST, &m_product_rn_list);
            Env.AddRef(_PARAM_NAME_BEHAVIOR_LIST, &m_customer_rating_list);
            Env.AddRef(_PARAM_NAME_PERS_RESULT_CODE, &m_result_code);
			Env.AddRef(_PARAM_NAME_LIKERT_DIMENSION_NAME, &likert_dimension);
			Env.AddRef(_PARAM_NAME_INTEREST_DIMENSION_NAME, &interest_dimension);
			Env.AddRef(_PARAM_NAME_LIKERT_RATING_STRATEGY, &likert_rating_strategy);
			Env.AddRef(_PARAM_NAME_INTEREST_RATING_STRATEGY, &interest_rating_strategy);			

            Err = NC_ApiManager::GetUniqueInstance().ProcessApi(Req, Res, Env,
                                                                _TASK_PROFILE_UPD_PROD,
                                                                m_merchant_rn );

            Env.Remove(_PARAM_NAME_MERCHANT_REF_NUM);
            Env.Remove(_PARAM_NAME_PRODUCT_REF_NUM_LIST);
            Env.Remove(_PARAM_NAME_BEHAVIOR_LIST);
            Env.Remove(_PARAM_NAME_PERS_RESULT_CODE);
            Env.Remove(_PARAM_NAME_LIKERT_DIMENSION_NAME);
            Env.Remove(_PARAM_NAME_INTEREST_DIMENSION_NAME);
            Env.Remove(_PARAM_NAME_LIKERT_RATING_STRATEGY);
            Env.Remove(_PARAM_NAME_INTEREST_RATING_STRATEGY);                   

            if (Err == &_ERR_CMD_ERR_HANDLED)  // API already handled the error!
            {
                throw Err;
            }

            if (Err != NULL) // Unhandled error
            {
                error << indent;
                error.nls(&_ERR_UNHANDLED_IN_API,__FILE__,__LINE__) << endl;
                return false;
            } 

            //           
            // If personalization error occurs, then the result code will have "pers_error".  
            //
            if (m_result_code == "pers_error")
            {
               debug << indent; 
               status.nls(&_NP_ERR_PERSONALIZATION_ENGINE) << endl;
               status << indent << "Redirect to [" << errorURL << "]\n";
               Res.setLocation(errorURL, true);
            }
        }
        else
        {
            error << indent; 
            error.nls(&_API_PERS_ENGINE_ERROR, msgPersonalizationDisabled) << endl;
            m_result_code = "disabled";
        }
            
        // Clean up on the way out. . .
        m_product_rn_list.Clean(true);
        m_customer_rating_list.Clean(true);

        // Set return code
        redirectNVP.Add(paramNameResultCode, m_result_code);
            
        Res.Redirect(Req, paramNameURL, redirectNVP);

        return true;
    }
};

ClassName RateProduct::_STR_ThisClass("RateProduct");

static bool Z = NC_CommandManager::GetUniqueInstance().RegisterCommand("IBM",
                                                                       "NC",
                                                                       "RateProduct",
                                                                       1.0,
                                                                       new RateProduct);



