/****************************************************************************************/
/*                                                                                      */
/* IBM Confidential                                                                     */
/*                                                                                      */
/* Net.Commerce Source Materials                                                        */
/*                                                                                      */
/* (C) Copyright IBM Corp. 1998.                                                        */
/*                                                                                      */
/* The source code for this program is not published or otherwise divested of its trade */
/* secrets, irrespective of what has been deposited with the U.S. Copyright Office.     */
/*                                                                                      */
/****************************************************************************************/
//////////////////////////////////////////////////////////////////////////////////////////
// Change History
//////////////////////////////////////////////////////////////////////////////////////////
// YYMMDD  F/D#   WHO    Description
// ---------------------------------------------------------------------------------------
// 
//////////////////////////////////////////////////////////////////////////////////////////

#include "nc_core.pch"
#include "api/api.pch"
#include <containers/ldh_list.h>
#include "commands/recommend_product.h"
#include "commands/np_configuration.h"
#include "messages/np_msg.pch"
#include "messages/np_msg_src.h"

/****************************************************************************************/
#ifdef __TRACE_COMMANDS__
  typedef TraceYes Trace;
#else
  typedef TraceNo  Trace;
#endif
static Trace trace("NC_COMMANDS ("__FILE__")");
/****************************************************************************************/

//////////////////////////////////////////////////////////////////////////////////////////
//
// Internal constants
//

//=========================================================
// External Parameters -- interface with URL
//=========================================================

static const StringWithOwnership paramNameMerchantRefNum("merchant_rn");
static const StringWithOwnership paramNameFromRank  ("from_rank");
static const StringWithOwnership paramNameToRank    ("to_rank");
static const StringWithOwnership paramNameResultCode("rate_result");
static const StringWithOwnership paramNameURL("url");


//=========================================================
// Internal Parameters -- interface with NCEnvironment
//=========================================================
static const StringWithOwnership _PARAM_NAME_PERS_RESULT_CODE("PERS_RESULT_CODE");
static const StringWithOwnership _PARAM_NAME_FROM_RANK("FROM_RANK");
static const StringWithOwnership _PARAM_NAME_TO_RANK("TO_RANK");
static const StringWithOwnership _PARAM_NAME_LIKERT_DIMENSION_NAME("LIKERT_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_INTEREST_DIMENSION_NAME("INTEREST_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_LIKERT_STRATEGY("LIKERT_PREDICTION_STRATEGY");
static const StringWithOwnership _PARAM_NAME_INTEREST_STRATEGY("INTEREST_PREDICTION_STRATEGY");

//=========================================================
// Others
//=========================================================

static const FuncName _STR_Constructor("constructor");
static const char *msgPersonalizationDisabled = "Personalization not enabled";

static const StringWithOwnership _TASK_GET_REC_PROD("GL_GET_REC_PROD");
//static const StringWithOwnership _STR_RECOMMENDED_PRODUCTS_MACRO_NAME("/demomall/rcmnd1.d2w");

// HTML page to display if personalization not enabled
static const StringWithOwnership naURL("/demomall/pers_na.htm");

// HTML page to display if personalization error occurs
static const StringWithOwnership errorURL("/demomall/pers_error.htm");


//////////////////////////////////////////////////////////////////////////////////////////
//
//  Command RecommendProduct
//



bool RecommendProduct::CheckParameters( const HttpRequest& Req,
                                              HttpResponse& Res,
                                              NC_Environment& Env,
                                              NC_Environment& Resources )
{
    if (Misc::CheckFieldExistenceLong(Req, paramNameMerchantRefNum,
                                      _merchant_rn, false) == false)  return false;

    // Optional parameters
    Misc::CheckFieldExistenceLong(Req, paramNameFromRank, _from_rank, false);
    Misc::CheckFieldExistenceLong(Req, paramNameToRank, _to_rank, false);
	
    return true;
}

bool RecommendProduct::Process( const HttpRequest& Req,
                                      HttpResponse& Res,
                                      NC_Environment& Env )
{
    Trace::Tracer T(_STR_Process, _STR_ThisClass);


    if (Misc::CheckConfigurationVariable(ConfField::_CONF_PERS_ENABLED, true) == "YES") 
    {
        const ErrorMsg_Cmd* Err;

			// Dimension to be used
			String likert_dimension;
			likert_dimension.Clean();
			likert_dimension << LIKERT_DIMENSION_NAME  << _merchant_rn;

			String interest_dimension;
			interest_dimension.Clean();
			interest_dimension << INTEREST_DIMENSION_NAME << _merchant_rn;
			
			String likert_prediction_strategy;
			likert_prediction_strategy.Clean();
			likert_prediction_strategy << LIKERT_PREDICTION_STRATEGY << _merchant_rn;
 
			String interest_prediction_strategy;
			interest_prediction_strategy.Clean();
			interest_prediction_strategy << INTEREST_PREDICTION_STRATEGY << _merchant_rn;

        Env.AddRef(_PARAM_NAME_MERCHANT_REF_NUM, &_merchant_rn);
        Env.AddRef(_PARAM_NAME_FROM_RANK, &_from_rank); 
        Env.AddRef(_PARAM_NAME_TO_RANK, &_to_rank);
        Env.AddRef(_PARAM_NAME_PRODUCT_REF_NUM_LIST, &_product_refnum_list);
        Env.AddRef(_PARAM_NAME_PERS_RESULT_CODE, &_result_code);
		Env.AddRef(_PARAM_NAME_LIKERT_DIMENSION_NAME, &likert_dimension);
		Env.AddRef(_PARAM_NAME_INTEREST_DIMENSION_NAME, &interest_dimension);
     	Env.AddRef(_PARAM_NAME_LIKERT_STRATEGY, &likert_prediction_strategy);
		Env.AddRef(_PARAM_NAME_INTEREST_STRATEGY, &interest_prediction_strategy);

    
        Err = NC_ApiManager::GetUniqueInstance().ProcessApi( Req, Res, Env,
                                                          _TASK_GET_REC_PROD,
                                                          _merchant_rn );

        Env.Remove(_PARAM_NAME_MERCHANT_REF_NUM);
        Env.Remove(_PARAM_NAME_FROM_RANK);
        Env.Remove(_PARAM_NAME_TO_RANK);
        Env.Remove(_PARAM_NAME_PRODUCT_REF_NUM_LIST);
        Env.Remove(_PARAM_NAME_PERS_RESULT_CODE);
        Env.Remove(_PARAM_NAME_LIKERT_DIMENSION_NAME);
        Env.Remove(_PARAM_NAME_INTEREST_DIMENSION_NAME);
        Env.Remove(_PARAM_NAME_LIKERT_STRATEGY);
        Env.Remove(_PARAM_NAME_INTEREST_STRATEGY);
 

        if (Err == &_ERR_CMD_ERR_HANDLED)  // API already handled the error!
        {
            throw Err;
        }

        if (Err != NULL) // Unhandled error
        {
            error << indent;
            error.nls(&_ERR_UNHANDLED_IN_API,__FILE__,__LINE__) << endl;
            return false;
        } 


        //           
        // If personalization error occurs, then the result code will have "pers_error".  Exit and
        // display the personalization error page.
        //
        if (_result_code == "pers_error")
        {
           debug << indent; 
           status.nls(&_NP_ERR_PERSONALIZATION_ENGINE) << endl;
           debug << indent << "Redirect to [" << naURL << "]\n";
           Res.setLocation(errorURL, true);
           return true;
        }




        if (!_product_refnum_list.Empty())
        {
           TemplateNC_PList<String, String>::Iterator lvI = _product_refnum_list.getIterator();
           lvI.Begin();
           long irec = 1;
           NVPHash &macro_nvp = Misc::GetMacroNVPHash(Env);
           
           while (lvI.getElement() != NULL)
           {
              String parmKey("ProdRec");
              parmKey << irec;
              String prodRefNum = lvI.getElement();
              if (!prodRefNum.IsEmpty())
                 macro_nvp.Add(parmKey, prodRefNum);
              else
                 break;
              
              ++irec;         
              lvI.getNextElement();
           }  // while
           _product_refnum_list.Clean(true);
        }  

        const StringWithOwnership ViewTaskName("REC_PROD_DSP");
      // Set return code
        Env.Add(paramNameResultCode, _result_code);
		Env.Add(paramNameMerchantRefNum,_merchant_rn);
 
        //return Misc::SetView(Env, ViewTaskName, _merchant_rn, _STR_RECOMMENDED_PRODUCTS_MACRO_NAME);
	    return Misc::SetView(Env, ViewTaskName, _merchant_rn); 	

    }  // PERS_ENABLED
    else {
        error << indent; 
        error.nls(&_API_PERS_ENGINE_ERROR, msgPersonalizationDisabled) << endl;
        _result_code = "disabled";

        debug << indent << "Redirect to [" << naURL << "]\n";

        Res.setLocation(naURL, true);
        return true;
    }
}

static bool Z = 
   NC_CommandManager::GetUniqueInstance().RegisterCommand( "IBM",
                                                           "NC",
                                                           "RecommendProduct",
                                                           1.0,
                                                           new RecommendProduct );
                                                        
