/****************************************************************************************/
/*                                                                                      */
/* IBM Confidential                                                                     */
/*                                                                                      */
/* Net.Commerce Source Materials                                                        */
/*                                                                                      */
/* (C) Copyright IBM Corp. 1998.                                                        */
/*                                                                                      */
/* The source code for this program is not published or otherwise divested of its trade */
/* secrets, irrespective of what has been deposited with the U.S. Copyright Office.     */
/*                                                                                      */
/****************************************************************************************/
//////////////////////////////////////////////////////////////////////////////////////////
// Change History
//////////////////////////////////////////////////////////////////////////////////////////
// YYMMDD  F/D#   WHO    Description
// ---------------------------------------------------------------------------------------
// 980929         jbrains Initial version
//////////////////////////////////////////////////////////////////////////////////////////

#include "nc_core.pch"
#include "api/api.pch"
#include <containers/ldh_list.h>
#include "commands/np_configuration.h"
#include "messages/np_msg.pch"
#include "messages/np_msg_src.h"

/****************************************************************************************/
#ifdef __TRACE_COMMANDS__
  typedef TraceYes Trace;
#else
  typedef TraceNo  Trace;
#endif
static Trace trace("NC_COMMANDS ("__FILE__")");
/****************************************************************************************/


//////////////////////////////////////////////////////////////////////////////////////////
//
// Internal constants
//

//====================================================================
// External Parameters -- interface with URL
//====================================================================
static const StringWithOwnership paramNameMerchantRefNum("merchant_rn");
static const StringWithOwnership paramNameProductRefNum("product_rn");
static const StringWithOwnership paramNameAvailable("available");
static const StringWithOwnership paramNameURL("url");
static const StringWithOwnership paramNameResultCode("perscenter_result");

//============================================================================
// Internal Parameters -- interface with NCEnvironment
//============================================================================
static const StringWithOwnership _PARAM_NAME_BEHAVIOR_LIST("BEHAVIOR_LIST");
static const StringWithOwnership _PARAM_NAME_PERS_RESULT_CODE("PERS_RESULT_CODE");
static const StringWithOwnership _PARAM_NAME_LIKERT_DIMENSION_NAME("LIKERT_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_INTEREST_DIMENSION_NAME("INTEREST_DIMENSION_NAME");
static const StringWithOwnership _PARAM_NAME_LIKERT_RATING_STRATEGY("LIKERT_RATING_STRATEGY");
static const StringWithOwnership _PARAM_NAME_INTEREST_RATING_STRATEGY("INTEREST_RATING_STRATEGY");
static const StringWithOwnership _PARAM_NAME_RECOMMENDATION_STATUS("RECOMMENDATION_STATUS");

//static const StringWithOwnership _TASK_REMOVE_PRODUCT_RATING("REMOVE_PRODUCT_RATINGS");

//====================================================================
// Others
//====================================================================
static const StringWithOwnership _TASK_REMOVE_PRODUCT_RATING("GL_REMOVE_PROD_RATINGS");

static const FuncName _STR_Constructor("<constructor>");

static const StringWithOwnership _STR_0("0");
static const StringWithOwnership _STR_1("1");
static const StringWithOwnership _STR_2("2");
static const StringWithOwnership _STR_YES("yes");
static const StringWithOwnership _STR_NO("no");

// HTML page to display if personalization error occurs
static const StringWithOwnership errorURL("/demomall/pers_error.htm");

//////////////////////////////////////////////////////////////////////////////////////////
//
//  Command SynchRecommendationDatabase
//

class RemoveProdRatings : public NC_Command
{
private:
    static ClassName _STR_ThisClass;

    String m_merchant_rn;
    String m_product_rn;
    String m_available;
    String m_redirect_url;
    String m_result_code;
    bool m_url_specified;

public:
    RemoveProdRatings() : m_merchant_rn(), m_product_rn(), m_available(),
                                    m_redirect_url(), m_url_specified(false)
    {
        Trace::Tracer T(_STR_Constructor, _STR_ThisClass);
        // Do nothing!
    }

    ~RemoveProdRatings()
    {
        // Do nothing!
    }

    void operator delete( void* p )
    {
        ::delete p;
    }

public:
    virtual bool CheckParameters(const HttpRequest& Req,
                                 HttpResponse& Res,
                                 NC_Environment& Env,
                                 NC_Environment& Resources)
    {
        Trace::Tracer T(_STR_CheckParameters, _STR_ThisClass);

        if(Misc::CheckFieldExistenceLong(Req, paramNameMerchantRefNum,
                                         m_merchant_rn, false) == false)  return false;

        long theMerchantRefNum;
        m_merchant_rn.getVal(theMerchantRefNum);
        if(theMerchantRefNum < 0)
        {
            error << indent;
            error.nls(&_ERR_CMD_INVALID_PARAM, paramNameMerchantRefNum.c_str()) << endl;
            return false;
        }

        if(Misc::CheckFieldExistenceLong(Req, paramNameProductRefNum,
                                         m_product_rn, false) == false)  return false;

        long theProductRefNum;
        m_product_rn.getVal(theProductRefNum);
        if(theProductRefNum < 0)
        {
            error << indent;
            error.nls(&_ERR_CMD_INVALID_PARAM, paramNameProductRefNum.c_str()) << endl;
            return false;
        }

        if(Misc::CheckFieldExistenceLong(Req, paramNameAvailable,
                                         m_available, false) == false)  return false;
        long theStatus;
        m_available.getVal(theStatus);
        switch(theStatus)
        {
            case 0:
                m_available = _STR_NO;
                break;

            case 1:
                m_available = _STR_YES; // Not used
                break;

            default:
                error << indent;
                error.nls(&_ERR_CMD_INVALID_PARAM, paramNameAvailable.c_str()) << endl;
                return false;
        }

        m_redirect_url.Clean();
        m_url_specified = Misc::CheckFieldExistence(Req, paramNameURL, m_redirect_url, false);

        return true;
    }


    virtual bool Process(const HttpRequest& Req,
                         HttpResponse& Res,
                         NC_Environment& Env)
    {
        Trace::Tracer T(_STR_Process, _STR_ThisClass);
        static const String successDocument("<HTML><HEAD></HEAD><BODY><FONT size=\"+1\">"
         "Recommendation database <I>successfully</I> updated.</FONT></BODY></HTML>");

        static const String errorDocument("<HTML><HEAD></HEAD><BODY><FONT size=\"+1\">"
         "Recommendation database is currently not <I>accessible</I>. Please update the database manually.</FONT></BODY></HTML>");

        NVPHash redirectNVP;

        if (Misc::CheckConfigurationVariable(ConfField::_CONF_PERS_ENABLED, true) == "YES")
        {
            const ErrorMsg_Cmd* Err;

			// Remove ratings from both dimensions
			// Dimension to be used
			String likert_dimension;
			likert_dimension.Clean();
			likert_dimension << LIKERT_DIMENSION_NAME << m_merchant_rn;

			String interest_dimension;
			interest_dimension.Clean();
			interest_dimension << INTEREST_DIMENSION_NAME << m_merchant_rn;


            Env.AddRef(_PARAM_NAME_MERCHANT_REF_NUM, &m_merchant_rn);
            Env.AddRef(_PARAM_NAME_PRODUCT_REF_NUM_LIST, &m_product_rn);
            Env.AddRef(_PARAM_NAME_RECOMMENDATION_STATUS, &m_available);
			Env.AddRef(_PARAM_NAME_LIKERT_DIMENSION_NAME, &likert_dimension);
			Env.AddRef(_PARAM_NAME_INTEREST_DIMENSION_NAME, &interest_dimension);
            Env.AddRef(_PARAM_NAME_PERS_RESULT_CODE, &m_result_code);


            Err = NC_ApiManager::GetUniqueInstance().ProcessApi(Req, Res, Env,
                                                               _TASK_REMOVE_PRODUCT_RATING,
                                                               m_merchant_rn );

            Env.Remove(_PARAM_NAME_MERCHANT_REF_NUM);
            Env.Remove(_PARAM_NAME_PRODUCT_REF_NUM_LIST);
            Env.Remove(_PARAM_NAME_RECOMMENDATION_STATUS);
            Env.Remove(_PARAM_NAME_LIKERT_DIMENSION_NAME);
            Env.Remove(_PARAM_NAME_INTEREST_DIMENSION_NAME);
            Env.Remove(_PARAM_NAME_PERS_RESULT_CODE);

            if (Err == &_ERR_CMD_ERR_HANDLED)  // API already handled the error!
            {
                throw Err;
            }

            if (Err != NULL) // Unhandled error
            {
                error << indent;
                error.nls(&_ERR_UNHANDLED_IN_API,__FILE__,__LINE__) << endl;
                return false;
            }

            // Should check for incoming URL, and if none, set to "success document"
			if(m_url_specified == false) {
				if ((m_result_code == "pers_error") || (m_result_code == "pers_disabled"))
				{
                   debug << indent;
                   status.nls(&_NP_ERR_PERSONALIZATION_ENGINE) << endl;
				   status << indent << "Redirect to [" << errorURL << "]\n";
                   Res.setDocument(errorDocument);
               }
				else
				{
					Res.setDocument(successDocument);
				}
			}
            else
            {
				 // Set return code
				redirectNVP.Add(paramNameResultCode, m_result_code);
                Res.Redirect(Req, paramNameURL, redirectNVP);
            }
        }
        else
        {
                Res.setDocument(errorDocument);

       }

        return true;
    }

};

ClassName RemoveProdRatings::_STR_ThisClass("RemoveProdRatings");

static bool Z = NC_CommandManager::GetUniqueInstance().RegisterCommand("IBM",
                                                                       "NC",
                                                                       "RemoveProdRatings",
                                                                       1.0,
                                                                       new RemoveProdRatings);


